/**************************************************************************************
 * Copyright (C) 2006 Esper Team. All rights reserved.                                *
 * http://esper.codehaus.org                                                          *
 * ---------------------------------------------------------------------------------- *
 * The software in this package is published under the terms of the GPL license       *
 * a copy of which has been included with this distribution in the license.txt file.  *
 **************************************************************************************/
package net.esper.eql.spec;

import net.esper.util.MetaDefItem;

import java.util.List;
import java.util.LinkedList;

/**
 * Descriptor generated by INSERT-INTO clauses specified in expressions to insert the
 * results of statement as a stream to further statements.
 */
public class InsertIntoDesc implements MetaDefItem
{
    private final boolean isIStream;
    private final String eventTypeAlias;
    private List<String> columnNames;

    /**
     * Ctor.
     * @param isIStream is true if insert (new data) events are fed, or false for remove (old data) events are fed
     * @param eventTypeAlias is the event type alias name
     */
    public InsertIntoDesc(boolean isIStream, String eventTypeAlias)
    {
        this.isIStream = isIStream;
        this.eventTypeAlias = eventTypeAlias;
        columnNames = new LinkedList<String>();
    }

    /**
     * Returns true if insert (new data) events are fed, or false for remove (old data) events are fed.
     * @return true for insert stream, false for remove stream
     */
    public boolean isIStream()
    {
        return isIStream;
    }

    /**
     * Returns name of event type to use for insert-into stream.
     * @return event type alias name
     */
    public String getEventTypeAlias()
    {
        return eventTypeAlias;
    }

    /**
     * Returns a list of column names specified optionally in the insert-into clause, or empty if none specified.
     * @return column names or empty list if none supplied
     */
    public List<String> getColumnNames()
    {
        return columnNames;
    }

    /**
     * Add a column name to the insert-into clause.
     * @param columnName to add
     */
    public void add(String columnName)
    {
        columnNames.add(columnName);
    }
}
